/*
 * funny.c: Well, I put some stuff here and called it funny.  So sue me. 
 *
 * written by michael sandrof
 *
 * copyright(c) 1990 
 *
 * see the copyright file, or do a help ircii copyright 
 */


#include "irc.h"

#include "ircaux.h"
#include "hook.h"
#include "vars.h"
#include "funny.h"
#include "names.h"
#include "server.h"
#include "lastlog.h"
#include "ircterm.h"
#include "output.h"
#include "numbers.h"
#include "parse.h"
#include "status.h"
#include "misc.h"
#include "pluto.h"

static	char	*match_str = NULL;

static	int	funny_min;
static	int	funny_max;
static	int	funny_flags;
static  char    *mode_channel = NULL;
static  char    *names_channel = NULL;
static  int	ignore_mode = 0;

void
funny_match(stuff)
	char	*stuff;
{
	malloc_strcpy(&match_str, stuff);
}

void
funny_set_ignore_mode _((void)) 
{
	mode_channel = names_channel;
	names_channel = NULL;
	ignore_mode++;
}

int funny_is_ignore_channel _((void)) 
{
	return((names_channel != NULL));
}

void funny_set_ignore_channel (char *str)
{
	malloc_strcpy(&names_channel, str);
}


void
set_funny_flags(min, max, flags)
	int	min,
		max,
		flags;
{
	funny_min = min;
	funny_max = max;
	funny_flags = flags;
}

struct	WideListInfoStru
{
	char	*channel;
	int	users;
};

typedef	struct WideListInfoStru WideList;

static	WideList **wide_list = NULL;
static	int	wl_size = 0;
static	int	wl_elements = 0;

static	int	funny_widelist_users _((WideList **, WideList **));
static	int	funny_widelist_names _((WideList **, WideList **));

static	int
funny_widelist_users(left, right)
	WideList	**left,
			**right;
{
	if ((**left).users > (**right).users)
		return -1;
	else if ((**right).users > (**left).users)
		return 1;
	else
		return my_stricmp((**left).channel, (**right).channel);
}

static	int
funny_widelist_names(left, right)
	WideList	**left,
			**right;
{
	int	comp;

	if ((comp = my_stricmp((**left).channel, (**right).channel)) != 0)
		return comp;
	else if ((**left).users > (**right).users)
		return -1;
	else if ((**right).users > (**left).users)
		return 1;
	else
		return 0;
}


void funny_print_widelist()
{
	int	i;
	char	buffer1[BIG_BUFFER_SIZE];
	char	buffer2[BIG_BUFFER_SIZE];
	char	*ptr;

	if (!wide_list)
		return;

	if (funny_flags & FUNNY_NAME)
		qsort((void *) wide_list, wl_elements, sizeof(WideList *),
		(int (*) _((const void *, const void *))) funny_widelist_names);
	else if (funny_flags & FUNNY_USERS)
		qsort((void *) wide_list, wl_elements, sizeof(WideList *),
		(int (*) _((const void *, const void *))) funny_widelist_users);

	*buffer1 = '\0';
	for (i = 1; i < wl_elements; i++)
	{
		sprintf(buffer2, "%s(%d) ", wide_list[i]->channel,
				wide_list[i]->users);
		ptr = index(buffer1, '\0');
		if (strlen(buffer1) + strlen(buffer2) > CO - 5)
		{
			if (do_hook(WIDELIST_LIST, "%s", buffer1))
#ifdef WANT_ANSI
		put_it("%s", convert_output_format(get_string_var(FORMAT_WIDELIST_VAR), "%s %s", update_clock(GET_TIME), buffer1));
#else
				say("%s", buffer1);
#endif
			*buffer1 = '\0';
			strcat(buffer1, buffer2);
		}
		else
			strcpy(ptr, buffer2);
	}
	if (*buffer1 && do_hook(WIDELIST_LIST, "%s", buffer1))
#ifdef WANT_ANSI
		put_it("%s", convert_output_format(get_string_var(FORMAT_WIDELIST_VAR), "%s %s", update_clock(GET_TIME), buffer1));
#else
		say("%s" , buffer1);
#endif
	for (i = 0; i < wl_elements; i++)
	{
		new_free(&wide_list[i]->channel);
		new_free((char **)&wide_list[i]);
	}
	new_free((char **)&wide_list);
	wl_elements = wl_size = 0;
}

/*ARGSUSED*/
void
funny_list(from, ArgList)
	char	*from;
	char	**ArgList;
{
	char	*channel,
		*user_cnt,
		*line;
	WideList **new_list;
	int	cnt;
	static	char	format[25];
	static	unsigned int	last_width = -1;

	if (last_width != get_int_var(CHANNEL_NAME_WIDTH_VAR))
	{
#ifdef WANT_ANSI
		if ((last_width = get_int_var(CHANNEL_NAME_WIDTH_VAR)) != 0)
			sprintf(format, "%%s %%-%u.%us %%-5s %%s", /*thing_ansi,*/
				(unsigned char) last_width,
				(unsigned char) last_width);
		else
			sprintf(format, "%%s %%s %%-5s %%s"/*, thing_ansi*/);
#else
		if ((last_width = get_int_var(CHANNEL_NAME_WIDTH_VAR)) != 0)
			sprintf(format, "%s %%-%u.%us %%-5s  %%s", thing_ansi,
				(unsigned char) last_width,
				(unsigned char) last_width);
		else
			sprintf(format, "%s %%s\t%%-5s  %%s", thing_ansi);
#endif
	}
	channel = ArgList[0];
	user_cnt = ArgList[1];
	line = PasteArgs(ArgList, 2);
	if (funny_flags & FUNNY_TOPIC && !(line && *line))
			return;
	cnt = atoi(user_cnt);
	if (funny_min && (cnt < funny_min))
		return;
	if (funny_max && (cnt > funny_max))
		return;
	if ((funny_flags & FUNNY_PRIVATE) && (*channel != '*'))
		return;
	if ((funny_flags & FUNNY_PUBLIC) && (*channel == '*'))
		return;
	if (match_str)
	{
		if (wild_match(match_str, channel) == 0)
			return;
	}
	if (funny_flags & FUNNY_WIDE)
	{
		if (wl_elements >= wl_size)
		{
			new_list = (WideList **) new_malloc(sizeof(WideList *) *
			    (wl_size + 50));
			bzero(new_list, sizeof(WideList *) * (wl_size + 50));
			if (wl_size)
				bcopy(wide_list, new_list, sizeof(WideList *)
					* wl_size);
			wl_size += 50;
			new_free((char **)&wide_list);
			wide_list = new_list;
		}
		wide_list[wl_elements] = (WideList *)
			new_malloc(sizeof(WideList));
		wide_list[wl_elements]->channel = NULL;
		wide_list[wl_elements]->users = cnt;
		malloc_strcpy(&wide_list[wl_elements]->channel,
				(*channel != '*') ? channel : "Prv");
		wl_elements++;
		return;
	}
	if (do_hook(current_numeric, "%s %s %s %s", from,  channel, user_cnt,
	    line) && do_hook(LIST_LIST, "%s %s %s", channel, user_cnt, line))
	{
		if (channel && user_cnt)
		{
#ifdef WANT_ANSI
	put_it("%s", convert_output_format(get_string_var(FORMAT_LIST_VAR),"%s %s %s %s", update_clock(GET_TIME), *channel == '*'?"Prv":channel, user_cnt, stripansicodes(line)));
#else
			if (*channel == '*')
				put_it(format, "Prv", user_cnt, line);
			else
				put_it(format, channel, user_cnt, line);
#endif
		}
	}
}


void
funny_namreply(from, Args)
	char	*from;
	char	**Args;
{
	char	*type,
		*channel;
	static	char	format[40];
	static	unsigned int	last_width = -1;
	int	cnt;
	char	*ptr;
	char	*line, *t;
	int user_count = 0;
	PasteArgs(Args, 2);
	type = Args[0];
	channel = Args[1];
	line = Args[2];
	message_from(channel, LOG_CRAP);
	if (names_channel && channel && !my_stricmp(names_channel, channel))
	{
		for (t = line; *t; t++)
		{
			if (*t == ' ')
				user_count++;
		}
		if (get_int_var(SHOW_CHANNEL_NAMES_VAR) && 
			do_hook(current_numeric, "%s %s %s %s", from, type, channel,line))
		{
#ifdef WANT_ANSI
	put_it("%s", convert_output_format(get_string_var(FORMAT_NAMES_VAR), "%s %s %d",update_clock(GET_TIME), channel, user_count));
	if (*line)
	{	
		int count = 0;
		char *buffer = NULL;
		char special = '\0';
		t = next_arg(line, &line);
		if (get_string_var(FORMAT_NAMES_BANNER_VAR))
			malloc_strcpy(&buffer, get_string_var(FORMAT_NAMES_BANNER_VAR));
		do {
			if (*t == '@' || *t == '+' || *t == '~' || *t == '-')
			{
				special = *t;
				if (special == '+')
				malloc_strcat(&buffer, convert_output_format(get_string_var(FORMAT_NAMES_VOICECOLOR_VAR),"%c %s", special, ++t));
				else
				malloc_strcat(&buffer, convert_output_format(get_string_var(FORMAT_NAMES_OPCOLOR_VAR),"%c %s", special, ++t));
			}
			else
				malloc_strcat(&buffer, convert_output_format(get_string_var(FORMAT_NAMES_NICKCOLOR_VAR), "$ %s", t));
			malloc_strcat(&buffer, " ");
			if (count++ == 4)
			{
				put_it(buffer);
				if (get_string_var(FORMAT_NAMES_BANNER_VAR))
					malloc_strcpy(&buffer, get_string_var(FORMAT_NAMES_BANNER_VAR));
				else
					new_free(&buffer);
				count = 0;
			}
		} while ((t = next_arg(line, &line)));

		if (buffer)
			put_it(buffer);
		new_free(&buffer);
	}
		}
#else
			say("Users on %s: %s", channel, line);
#endif
		message_from(NULL, LOG_CURRENT);
		return;
	}
	if (last_width != get_int_var(CHANNEL_NAME_WIDTH_VAR))
	{
		if ((last_width = get_int_var(CHANNEL_NAME_WIDTH_VAR)) != 0)
			sprintf(format, "%%s: %%-%u.%us %%s",
				(unsigned char) last_width,
				(unsigned char) last_width);
		else
			strcpy(format, "%s: %s\t%s");
	}
	ptr = line;
	for (cnt = -1; ptr; cnt++)
	{
		if ((ptr = index(ptr, ' ')) != NULL)
			ptr++;
	}
	if (funny_min && (cnt < funny_min))
		return;
	else if (funny_max && (cnt > funny_max))
		return;
	if ((funny_flags & FUNNY_PRIVATE) && (*type == '='))
		return;
	if ((funny_flags & FUNNY_PUBLIC) && (*type == '*'))
		return;
	if (type && channel)
	{
		if (match_str)
		{
			if (wild_match(match_str, channel) == 0)
				return;
		}
		if (do_hook(current_numeric, "%s %s %s %s", from, type, channel,
			line) && do_hook(NAMES_LIST, "%s %s", channel, line))
		{
#ifdef WANT_ANSI
	put_it("%s", convert_output_format(get_string_var(FORMAT_NAMES_VAR), "%s %s %d", update_clock(GET_TIME), channel, cnt));
	if (*line)
	{	
		int count = 0;
		char *buffer = NULL;
		char special = '\0';
		t = next_arg(line, &line);
		if (get_string_var(FORMAT_NAMES_BANNER_VAR))
			malloc_strcpy(&buffer, get_string_var(FORMAT_NAMES_BANNER_VAR));
		do {
			if (*t == '@' || *t == '+' || *t == '~' || *t == '-')
			{
				special = *t;
				if (special == '+')
				malloc_strcat(&buffer, convert_output_format(get_string_var(FORMAT_NAMES_VOICECOLOR_VAR),"%c %s",(char )special, ++t));
				else
				malloc_strcat(&buffer, convert_output_format(get_string_var(FORMAT_NAMES_OPCOLOR_VAR),"%c %s",(char )special, ++t));
			}
			else
				malloc_strcat(&buffer, convert_output_format(get_string_var(FORMAT_NAMES_NICKCOLOR_VAR), "%c %s", (char )'$', t));
			malloc_strcat(&buffer, " ");
			if (count++ == 4)
			{
				put_it(buffer);
				if (get_string_var(FORMAT_NAMES_BANNER_VAR))
					malloc_strcpy(&buffer, get_string_var(FORMAT_NAMES_BANNER_VAR));
				else
					new_free(&buffer);
				count = 0;
			}
		} while ((t = next_arg(line, &line)));

		if (buffer)
			put_it(buffer);
		new_free(&buffer);
	}
#else
			switch (*type)
			{
			case '=':
				if (last_width &&(strlen(channel) > last_width))
				{
					channel[last_width-1] = '>';
					channel[last_width] = (char) 0;
				}
				put_it(format, "Pub", channel, line);
				break;
			case '*':
				put_it(format, "Prv", channel, line);
				break;
			case '@':
				put_it(format, "Sec", channel, line);
				break;
			}
#endif
		}
	}
	message_from(NULL, LOG_CURRENT);
}

void
funny_mode(from, ArgList)
	char	*from,
		**ArgList;
{
	char	*mode, *channel;
	ChannelList *chan;
	
	if (!ArgList[0]) return;
	channel = ArgList[0];
	mode = ArgList[1];
	PasteArgs(ArgList, 1);
	if (ignore_mode) 
	{
		chan = prepare_command(&from_server, channel, NO_OP);
		update_channel_mode(from, channel ? channel : mode_channel, from_server, mode, chan);
		update_all_status();
		new_free(&mode_channel);
		ignore_mode--;
		check_sync(324, channel, NULL, NULL, NULL, chan);
	}
	else
	{
		if (channel)
		{
			message_from(channel, LOG_CRAP);
			if (do_hook(current_numeric, "%s %s %s", from, channel, mode))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_MODE_CHANNEL_VAR), "%s %s %s %s %s", update_clock(GET_TIME), from, *FromUserHost ? FromUserHost:"", channel, mode));
#else
				put_it("%s Mode for channel %s is \"%s\"",
					numeric_banner(), channel, mode);
#endif
		}
		else
		{
			if (do_hook(current_numeric, "%s %s", from, mode))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_MODE_CHANNEL_VAR), "%s %s %s %s %s", update_clock(GET_TIME), from, *FromUserHost ? FromUserHost:"", "", mode, ""));
#else
				put_it("%s Channel mode is \"%s\"",
					numeric_banner(), mode);
#endif
		}
	}
}

void
update_user_mode(modes)
	char	*modes;
{
	int	onoff = 1;

	while (*modes)
	{
		switch(*modes++)
		{
		case '-':
			onoff=0;
			break;
		case '+':
			onoff=1;
			break;
		case 'b':
		case 'B':
			set_server_flag(from_server, USER_MODE_B, onoff);
			break;
		case 'd':
		case 'D':
			set_server_flag(from_server, USER_MODE_D, onoff);
			break;
		case 'l':
		case 'L':
			set_server_flag(from_server, USER_MODE_L, onoff);
			break;
		case 'r':
		case 'R':
			set_server_flag(from_server, USER_MODE_R, onoff);
			break;
		case 'o':
		case 'O':
			set_server_operator(from_server, onoff);
			break;
		case 's':
		case 'S':
			set_server_flag(from_server, USER_MODE_S, onoff);
			break;
		case 'i':
		case 'I':
			set_server_flag(from_server, USER_MODE_I, onoff);
			break;
		case 'w':
		case 'W':
			set_server_flag(from_server, USER_MODE_W, onoff);
			break;
		case 'f':
		case 'F':
			set_server_flag(from_server, USER_MODE_F, onoff);
			break;
		case 'u':
		case 'U':
			set_server_flag(from_server, USER_MODE_U, onoff);
			break;
		case 'c':
		case 'C':
			set_server_flag(from_server, USER_MODE_C, onoff);
			break;
		case 'k':
		case 'K':
			set_server_flag(from_server, USER_MODE_K, onoff);
			break;
		}
	}
}

extern void	reinstate_user_modes _((void))
{
	char *modes = get_umode(from_server);
	if (modes && *modes)
		send_to_server("MODE %s +%s", get_server_nickname(from_server), modes);
}
