/*
 * notice.c: special stuff for parsing NOTICEs
 *
 * Written By Michael Sandrof
 *
 * Copyright(c) 1991
 *
 * See the COPYRIGHT file, or do a HELP IRCII COPYRIGHT 
 */
#define COMPAT_27

#include "irc.h"

#include "whois.h"
#include "ctcp.h"
#include "window.h"
#include "lastlog.h"
#include "flood.h"
#include "vars.h"
#include "ircaux.h"
#include "hook.h"
#include "ignore.h"
#include "server.h"
#include "funny.h"
#include "output.h"
#include "names.h"
#include "parse.h"
#include "notify.h"
#include "pluto.h"
#include "misc.h"
#include "status.h"

extern	char	*FromUserHost;
static	void	parse_server_notice _((char *, char *));
extern void reload_save _((char *, char *, char *));

int	doing_notice = 0;
int	oper_kills = 0;
int	nick_collisions = 0;

static	void
parse_server_notice(from, line)
	char	*from,
		*line;
{
	int	lastlog_level = 0;
	int	flag = 0;
	Window	*old_to_window = to_window;
			
	char *fr, *for_, *temp, *temp2, *server, *server2;
	fr = for_ = temp = temp2 = server = server2 = NULL;
	
	if (!from || !*from)
		from = server_list[from_server].itsname ?
			server_list[from_server].itsname :
			server_list[from_server].name;
#ifdef COMPAT_27
	if (get_int_var(SUPPRESS_SERVER_MOTD_VAR) &&
		get_server_motd(from_server))
	{
		if (strncmp("*** Message-of-today", line, 20) == 0)
		{
			set_server_motd(from_server, 0);
			return;
		}
		if (strncmp("MOTD ", line, 5) == 0)
		{
			set_server_motd(from_server, 1);
			return;
		}
		if (strcmp("* End of /MOTD command.", line) == 0)
		{
			set_server_motd(from_server, 0);
			return;
		}
	}
        else if (!get_server2_8(from_server) && (!strncmp(line, "*** Your host is ", 17)))
                got_initial_version(line);
	else
#endif	
	if (!strncmp(line, "*** Notice --", 13))
	{
		message_from(NULL, LOG_OPNOTE);
		lastlog_level = set_lastlog_msg_level(LOG_OPNOTE);
	}

	message_from(NULL, LOG_SNOTE);
	lastlog_level = set_lastlog_msg_level(LOG_SNOTE);
	
	if (*line != '*'  && *line != '#' && strncmp(line, "MOTD ", 4))
		flag = 1;
	else
		flag = 0;

  
	split_kill(line, &fr, &temp2,
			&server, &server2, &for_, &temp); 
	if (!server)
		server = "Local";
	if (!server2)
		server2 = "Local";
	if (!temp2)
		temp2 = "UNKNOWN";
	if (!temp)
		temp = "()";	
	if (!fr)
		fr = "unknown";
	if (!for_)
		for_ = "unknown";
	
	if (strstr(line, "Received KILL message for") && fr && for_)
	{
		if (get_int_var(OV_VAR))
			to_window = get_window_by_name("OV");
		if ((strncmp(server, "Local",5) == 0))
		{
			if (do_hook(SERVER_NOTICE_LOCAL_KILL_LIST, "%s %s %s %s", for_, server, fr, temp2, temp))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_SERVER_MSG1_VAR), "%s Local Kill %s %s %s %s %s", update_clock(GET_TIME), for_, server, fr, temp2, temp));
#else
				say("Local Ircop Kill for %s by %s %s (%s %s)", for_, fr, temp2, server, temp);
#endif
		} else {
			if (do_hook(SERVER_NOTICE_FOREIGN_KILL_LIST, "%s %s %s %s", for_, server, fr, temp2, temp))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_SERVER_MSG1_VAR), "%s Foreign Kill %s %s %s %s %s", update_clock(GET_TIME), for_, server, fr, temp2, temp));
#else
				say("Foreign Ircop Kill for %s by %s %s (%s %s)", for_, fr, temp2, server, temp);
#endif
		}
		if (get_int_var(OV_VAR))
			to_window = old_to_window;
		oper_kills++;
		update_all_status();
 	} 
#if 0
 	else if (strstr(line, "Nick change collision") && fr && for_)
	{
		if (!get_int_var(SHOW_SERVER_KILLS_VAR))
			goto done;  	
		if (get_int_var(OV_VAR))
			to_window = get_window_by_name("OV");
		if (do_hook(SERVER_NOTICE_SERVER_KILL_LIST,"%s %s %s %s", for_, fr, temp2, server, temp))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_SERVER_MSG1_VAR), "%s Nick Collision %s %s %s %s %s", update_clock(GET_TIME), for_, server, fr, temp2, temp));
#else
			put_it(flag ? "%s %s" : "%s%s", flag ? thing_ansi:"", line);
#endif
		if (get_int_var(OV_VAR))
			to_window = old_to_window;
		nick_collisions++;
		update_all_status();
	}
	else if (strstr(line, "Nickname collision") && fr && for_)
	{
		if (!get_int_var(SHOW_SERVER_KILLS_VAR))
			goto done;  	
		if (get_int_var(OV_VAR))
			to_window = get_window_by_name("OV");
		if (do_hook(SERVER_NOTICE_SERVER_KILL_LIST, "%s %s %s %s", for_, fr, temp2, server, temp))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_SERVER_MSG1_VAR), "%s Nickname Collision %s %s %s %s %s", update_clock(GET_TIME), for_, server, fr, temp2, temp));
#else
			put_it(flag ? "%s %s" : "%s%s", flag?thing_ansi:"", line);
#endif
		if (get_int_var(OV_VAR))
			to_window = old_to_window;
		nick_collisions++;
		update_all_status();
	}
  	else if (strstr(line, "Fake") && fr && for_)
  	{
		if (!get_int_var(SHOW_FAKES_VAR))
			goto done;  	
		/* SERVER_NOTICE_FAKES_MYCHANNEL_LIST */
		if (lookup_channel(fr, from_server, CHAN_NOUNLINK))
		{
			if (do_hook(SERVER_NOTICE_FAKES_MYCHANNEL_LIST, "%s %s %s %s", for_, fr, temp2, server, temp))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_SERVER_MSG1_VAR), "%s Fake Msg %s %s %s %s %s", update_clock(GET_TIME), for_, server, fr, temp2, temp));
#else
  				put_it(flag ? "%s %s" : "%s%s", flag? thing_ansi : "", line);
#endif
		}
		else 
		{
			if (do_hook(SERVER_NOTICE_FAKES_LIST, "%s %s %s %s", for_, fr, temp2, server, temp))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_SERVER_MSG1_VAR), "%s Notice Fake %s %s %s %s %s", update_clock(GET_TIME), for_, server, fr, temp2, temp));
#else
  				put_it(flag ? "%s %s" : "%s%s", flag?thing_ansi:"", line);
#endif
		}
  	}
  	else if (strstr(line, "Unauth"))
  	{
		if (!get_int_var(SHOW_UNAUTHS_VAR))
			goto done;  	
		if (do_hook(SERVER_NOTICE_LIST, flag ? "%s *** %s" : "%s %s", from, line))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_SERVER_MSG1_VAR), "%s Unauth Connect %s %s %s %s %s", update_clock(GET_TIME), for_, server, fr, temp2, temp));
put_it("%s", convert_output_format(get_string_var(FORMAT_SERVER_MSG1_VAR), "%s Unauth Connect %s %s %s", update_clock(GET_TIME), from, line));
#else
	               	put_it(flag ? "%s %s" : "%s%s", flag?thing_ansi:"", line);
#endif
	}
#endif
	else if (get_string_var(FORMAT_SERVER_NOTICE_BOT_VAR) && (strstr(line, "EggDrop alarm activated") || strstr(line, "JohBot alarm activated") || strstr(line, "Possible vlad/joh/com") || strstr(line, "Possible EggDrop") || strstr(line, "Possible bot")) )
	{
		char *temp_line = NULL;
		char *p, *nick, *q;
		if (!get_int_var(SHOW_BOTS_VAR) || !(p = strchr(line, ':')))
			goto done;
		malloc_strcpy(&temp_line, line);
		p = strchr(temp_line, ':');
		*p = 0; p+=2;
		nick = next_arg(p, &p);
		q = strchr(temp_line, '-');
		*q = 0; q+=3;
				
		if (do_hook(SERVER_NOTICE_LIST, flag ? "%s *** %s" : "%s %s", from, line))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_SERVER_NOTICE_BOT_VAR), "%s Warning %s %s %s %s", update_clock(GET_TIME), from, nick, p, q));
#else
	               	put_it(flag ? "%s %s" : "%s%s", flag?thing_ansi:"", line);
#endif
		
		new_free(&temp_line);
	} 
	else if (strstr(line, "is now operator"))
	{
		if (do_hook(SERVER_NOTICE_LIST, flag ? "%s *** %s" : "%s %s", from, line))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_OPER_VAR), "%s %s %s", update_clock(GET_TIME), from, line));
#else
	               	put_it(flag ? "%s %s" : "%s%s", flag?thing_ansi:"", line);
#endif
	}
	else
	{
		if (!get_int_var(SHOW_SERVER_CRAP_VAR))
			goto done;  	
		if (do_hook(SERVER_NOTICE_LIST, flag ? "%s *** %s" : "%s %s", from, line))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_SERVER_NOTICE_VAR), "%s %s %s", update_clock(GET_TIME), from, stripansicodes(line)));
#else
	               	put_it(flag ? "%s %s" : "%s%s", flag?thing_ansi:"", line);
#endif
	}
done:	
	if (lastlog_level)
	{
		set_lastlog_msg_level(lastlog_level);
		message_from(NULL, lastlog_level);
	}
}

void 
parse_notice(from, Args)
	char 	*from;
	char 	**Args;
{
	int	level,
		type;
	char	*to;
	int	no_flooding;
	int	flag;
	char	*high,
		not_from_server = 1;
	char	*line;
	
	ChannelList *tmpc;

		
	PasteArgs(Args, 1);
	to = Args[0];
	line = Args[1];
	if (!to || !line)
		return;
	doing_notice = 1;
	
	if (*to)
	{
		if (is_channel(to))
		{
			message_from(to, LOG_NOTICE);
			type = PUBLIC_NOTICE_LIST;
		}
		else
		{
			message_from(from, LOG_NOTICE);
			type = NOTICE_LIST;
		}
		tmpc = lookup_channel(to, from_server, CHAN_NOUNLINK);
		update_stats(NOTICELIST, to, from, tmpc, 0);		
		if (from && *from && strcmp(get_server_itsname(from_server), from))
		{
			int not_reply = 0;
			if (check_auto_reply(line))
				not_reply++;
			switch ((flag = check_ignore(from, FromUserHost, to, IGNORE_NOTICES)))
			{
				case IGNORED:
				{
					doing_notice = 0;
					return;
				}
				case HIGHLIGHTED:
					high = highlight_char;
					break;
				default:
					high = empty_string;
			}
		/*
		 * only dots in servernames, right ?
		 *
		 * But a server name doesn't nessicarily have to have
		 * a 'dot' in it..  - phone, jan 1993.
		 */
			if (index(from, '.'))
				not_from_server = 0;
			
			line = do_notice_ctcp(from, to, line);
			if (!line || !*line)
			{
				doing_notice = 0;
				return;
			}
			level = set_lastlog_msg_level(LOG_NOTICE);
			no_flooding = check_flooding(from, NOTICE_FLOOD, line, NULL);
			
			if (sed == 1)
			{
				if (do_hook(ENCRYPTED_NOTICE_LIST, "%s %s %s", from, to, line))
#ifdef WANT_ANSI
	put_it("%s", convert_output_format(get_string_var(FORMAT_ENCRYPTED_NOTICE_VAR), "%s %s %s %s", update_clock(GET_TIME), from, FromUserHost, stripansicodes(line)));
#endif
				sed = 0;
			}
			else
			{
				if (no_flooding)
				{
					if (type == NOTICE_LIST)
					{
						logmsg(0, "-", from, line, FromUserHost, 0);
						if (do_hook(type, "%s %s", from, line))
#ifdef WANT_ANSI
	put_it("%s", convert_output_format(get_string_var(FORMAT_NOTICE_VAR), "%s %s %s %s", update_clock(GET_TIME), from, FromUserHost, stripansicodes(line)));
#else
							put_it("%s-%s-%s %s", high, from, high, line);
#endif
					}
					else
					{
						if (do_hook(type, "%s %s %s", from, to, line))
#ifdef WANT_ANSI
	put_it("%s", convert_output_format(get_string_var(not_reply?FORMAT_PUBLIC_NOTICE_AR_VAR:FORMAT_PUBLIC_NOTICE_VAR), "%s %s %s %s %s", update_clock(GET_TIME), from, FromUserHost, to, stripansicodes(line)));
#else
							put_it("%s-%s:%s-%s %s", high, from, to, high, line);
#endif
						if (beep_on_level & LOG_NOTICE)
							beep_em(1);
					}
				}
/*
				malloc_strcpy(&last_notice, convert_output_format(get_string_var(FORMAT_RELN_VAR), "%s %s %s %s %s", update_clock(GET_TIME), from, FromUserHost, to, stripansicodes(line)));
*/
				add_last_type(&last_notice[0], from, FromUserHost, to, stripansicodes(line));
				set_lastlog_msg_level(level);
				if (not_from_server)
					notify_mark(from, 1, 0);
			}
		}
		else 
			parse_server_notice(from, line);
	}
	else
#ifdef WANT_ANSI
	put_it("%s", convert_output_format(get_string_var(FORMAT_SERVER_MSG2_VAR), "%s %s %s", update_clock(GET_TIME), from, stripansicodes(line+1)));
#else
		put_it("%s", stripansicodes(line + 1));
#endif
	doing_notice = 0;
}


void load_scripts(void)
{
	char buffer[BIG_BUFFER_SIZE+1];
	static int done = 0;
	if (!done++)
	{
		
		strcpy(buffer, "global");
		never_connected = 0;

		loading_global = 1;
		load("LOAD", buffer, empty_string);
		loading_global = 0;

		/* read the .ircrc file */
		if (access(bircrc_file, R_OK) == 0 && !quick_startup)
			load("LOAD", bircrc_file, empty_string);
		else if (access(ircrc_file, R_OK) == 0 && !quick_startup)
			load("LOAD", ircrc_file, empty_string);
		else if (get_int_var(NOVICE_VAR))
			say("If you have not already done so, please read the new user information with /HELP NEWUSER");
		if (!quick_startup)
			reload_save(NULL, NULL, NULL);
	}
	if (never_connected && server_list[from_server].away)
		send_to_server("AWAY :%s", server_list[from_server].away);
}

#ifdef COMPAT_27
/* got_initial_version: this is called when ircii get the NOTICE in
 * all version of ircd before 2.8, or the 002 numeric for 2.8 and
 * beyond.  I guess its handled rather badly at the moment....
 * added by phone, late 1992.
 */
void
got_initial_version(line)
	char	*line;
{
	char	server[81],
		sversion[21];
	char	*c;
	
	if ((sscanf(line, "*** Your host is %80s running version %20s",
			server, sversion)) != 2)
		return;
			
	server[strlen(server) - 1] = '\0';
	attempting_to_connect--;
	set_server_motd(from_server, 1);
	server_is_connected(from_server, 1);
	if ((c = (char *) index(server, '[')) != NULL)
		*c = '\0';	/*
				 * Handles the case where the server name is
				 * different to the host name.
				 */
	set_server_version(from_server, Server2_7);

	malloc_strcpy(&server_list[from_server].version_string, sversion);
	set_server_itsname(from_server, server);
	reconnect_all_channels(from_server);
	reinstate_user_modes(); 
	update_all_status();
	if (do_hook(CONNECT_LIST, "%s %d", get_server_name(from_server), get_server_port(from_server)))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_CONNECT_VAR), "%s %s %d", update_clock(GET_TIME), get_server_name(from_server), get_server_port(from_server)));
#else
		;
#endif	
}
#endif

/*
 * got_initial_version_28: this is called when ircii gets the serial
 * number 004 reply.  We do this becuase the 004 numeric gives us the
 * server name and version in a very easy to use fashion, and doesnt
 * rely on the syntax or construction of the 002 numeric.
 *
 * Hacked as neccesary by jfn, May 1995
 */
#ifdef __STDC__
extern void got_initial_version_28 (char **ArgList)
#else
extern void	got_initial_version_28(ArgList)
char	**ArgList;
#endif
{
	char *server, *sversion;

	server = ArgList[0];
	sversion = ArgList[1];
	
	attempting_to_connect--;
	set_server_motd(from_server, 1);
	server_is_connected(from_server, 1);
/*
	add_userhost_to_whois(get_server_nickname(from_server), got_my_userhost);
*/

	if (!strncmp(version, "2.8", 3))
	{
		if (strstr(version, "mu") || strstr(version, "me"))
			set_server_version(from_server, Server_u2_8);
		else
			set_server_version(from_server, Server2_8);
	}
	else if (!strncmp(version, "u2.9", 4))
		set_server_version(from_server, Server_u2_9);
	else if (!strncmp(version, "u2.10", 4))
		set_server_version(from_server, Server_u2_10);
	else if (!strncmp(version, "u3.0", 4))
		set_server_version(from_server, Server_u3_0);
	else
		set_server_version(from_server, Server2_8);

	malloc_strcpy(&server_list[from_server].version_string, sversion);
	set_server_itsname(from_server, server);
	reconnect_all_channels(from_server);
	reinstate_user_modes();

	update_all_status();
	do_hook(CONNECT_LIST, "%s %d", get_server_name(from_server), get_server_port(from_server));
}
