/* 
 * Copyright Colten Edwards (c) 1996
 */

#include "irc.h"
#include "server.h"
#include "edit.h"
#include "crypt.h"
#include "vars.h"
#include "ircaux.h"
#include "lastlog.h"
#include "window.h"
#include "screen.h"
#include "whois.h"
#include "hook.h"
#include "input.h"
#include "ignore.h"
#include "keys.h"
#include "names.h"
#include "alias.h"
#include "history.h"
#include "funny.h"
#include "ctcp.h"
#include "dcc.h"
#include "translat.h"
#include "output.h"
#include "exec.h"
#include "notify.h"
#include "numbers.h"
#include "status.h"
#include "list.h"
#include "pluto.h"
#include "bot.h"
#include "misc.h"

#define CHECK_FOR_OPS		/*
				 * * * * * * * define this if you want to
				 * * * * * * * make sure you are opped before
				 * * * * * * * doing a mode 
				 */
#define getrandom(min, max) ((rand() % (int)(((max)+1) - (min))) + (min))

char	*alias_special_char _((char **, char *, char *, char *, int *));



FILE *logptr = NULL;

int split_watch = 0;
int serv_action = 0;
int first_time = 0;

LastMsg last_msg[MAX_LAST_MSG] = { { NULL } };
LastMsg last_notice[MAX_LAST_MSG] = { { NULL } };
char *last_sent_msg = NULL;
char *last_sent_notice = NULL;
char *last_sent_topic = NULL;
char *last_sent_wall = NULL;
char *last_topic = NULL;
char *last_wall = NULL;

extern char *last_ctcp;
extern char *last_sent_ctcp;
extern int in_cparse;

#define SPLIT 1
#define FALSE 0

#ifndef TRUE
#define TRUE ~FALSE
#endif

void newhost(char *channel, char *arg, char *subargs);

extern char MyHostName[NAME_LEN+1];
extern char hostname[NAME_LEN+1];

extern struct in_addr MyHostAddr;	/*

					 * 
					 * * The local machine address 
					 */


extern Ignore *ignored_nicks;

char *color_str[] = {
	"[0;30m","[0;34m","[0;32m","[0;36m","[0;31m","[0;35m","[0;33m","[0m",
	"[1;30m","[1;34m","[1;32m","[1;36m","[1;31m","[1;35m","[1;33m","[1;37m", "[0m",
	"[0;40m", "[0;41m", "[0;42m","[0;43m", "[0;44m","[0;45m","[0;46m", "[0;46m","[7m", "[1m"};

irc_server *tmplink = NULL, *server_last = NULL;

#ifndef COLOR_TEST


int isme(char *nick)
{
	return ((my_stricmp(nick, get_server_nickname(from_server)) == 0) ? 1 : 0);
}

void clear_link(irc_server **serv1)
{
	irc_server *temp = *serv1, *hold;

	while (temp != NULL)
	{
		hold = temp->next;
		new_free(&temp->name);
		new_free(&temp->link);
		new_free(&temp->time);
		new_free((char **) &temp);
		temp = hold;
	}
	*serv1 = NULL;
}

void add_server(irc_server **serv1, char *channel, char *arg, int hops, char *time)
{
	irc_server *serv2;
	serv2 = (irc_server *) new_malloc(sizeof (irc_server));
	serv2->next = *serv1;
	malloc_strcpy(&serv2->name, channel);
	malloc_strcpy(&serv2->link, arg);
	serv2->hopcount = hops;
	serv2->time = m_strdup(time);
	*serv1 = serv2;
}

int find_server(irc_server *serv1, char *channel)
{
	irc_server *temp;

	for (temp = serv1; temp; temp = temp->next)
	{
		if (!my_stricmp(temp->name, channel))
			return TRUE;
	}
	return FALSE;
}

void parse_364(char *channel, char *args, char *subargs)
{
	if (!*channel || !*args)
		return;

	add_server(&tmplink, channel, args, atol(subargs), update_clock(GET_TIME));
}

void parse_365(char *channel, char *args, char *subargs)
{
	irc_server *serv1;

	for (serv1 = server_last; serv1; serv1 = serv1->next)
	{
		if (!find_server(tmplink, serv1->name))
		{
			if (!(serv1->status & SPLIT))
				serv1->status = SPLIT;
			if (serv1->count)
				continue;
			serv1->time = m_strdup(update_clock(GET_TIME));
			if (do_hook(LLOOK_SPLIT_LIST, "%s %s %d %s", serv1->name, serv1->link, serv1->hopcount, serv1->time))
#ifdef WANT_ANSI
	put_it("%s", convert_output_format(get_string_var(FORMAT_NETSPLIT_VAR), "%s %s %s %d", serv1->time, serv1->name, serv1->link, serv1->hopcount));
#else
				bitchsay("%s Split from %s %d", serv1->name, serv1->link, serv1->hopcount);
#endif
			serv1->count++;
		}
		else
		{
			if (serv1->status & SPLIT)
			{
				serv1->status = ~SPLIT;
				if (do_hook(LLOOK_JOIN_LIST, "%s %s %d", serv1->name, serv1->link, serv1->hopcount, serv1->time))
#ifdef WANT_ANSI
	put_it("%s", convert_output_format(get_string_var(FORMAT_NETJOIN_VAR), "%s %s %s %d", serv1->time, serv1->name, serv1->link, serv1->hopcount));
#else
					bitchsay("%s Merge %s %d", serv1->name, serv1->link, serv1->hopcount);
#endif
				serv1->count = 0;
			}
		}
	}
	for (serv1 = tmplink; serv1; serv1 = serv1->next)
	{
		if (!find_server(server_last, serv1->name)) {
			if (first_time == 1)
			{
				if (do_hook(LLOOK_ADDED_LIST, "%s %s %d", serv1->name, serv1->link, serv1->hopcount))
#ifdef WANT_ANSI
	put_it("%s", convert_output_format(get_string_var(FORMAT_NETADD_VAR), "%s %s %s %d", serv1->time, serv1->name, serv1->link, serv1->hopcount));
#else
					bitchsay("%s Added %s %d",serv1->name, serv1->link, serv1->hopcount);
#endif
				serv1->count = 0;
			}
			add_server(&server_last, serv1->name, serv1->link, serv1->hopcount, update_clock(GET_TIME));
		}
	}
	first_time = 1;
	clear_link(&tmplink);
}

/*
 * find split servers we hope 
 */
void linklook(char *channel, char *arg, char *subargs)
{
struct server_split *serv = server_last;
int count;
	if (serv == NULL)
	{
		bitchsay("Link Look not active!");
		return;
	}
	
	count = 0;
	while (serv)
	{
		if (serv->status & SPLIT)
		{
			if (!count)
#ifdef WANT_ANSI
	put_it("%s", convert_output_format(get_string_var(FORMAT_NETSPLIT_HEADER_VAR), "%s %s %s %s", "time","server","uplink","hops"));
#else
				say("distance		server			upLink");
#endif
			if (do_hook(LLOOK_SPLIT_LIST, "%s %s %d", serv->name, serv->link, serv->hopcount))
#ifdef WANT_ANSI
	put_it("%s", convert_output_format(get_string_var(FORMAT_NETSPLIT_VAR), "%s %s %s %d", serv->time, serv->name, serv->link, serv->hopcount));
#else
				say("[%3d] [%28s] <- [%s]", serv->hopcount, serv->name, serv->link);
#endif
			count++;
		}
		serv = serv->next;
	}
	if (count)
		bitchsay("There %s %d split servers", (count == 1) ? "is": "are", count);
	else
		bitchsay("No split servers found");
}

void send_last_type(char *channel, char *text)
{
	if (channel && is_channel(channel) && im_on_channel(channel))
		send_text(channel, stripansicodes(text), NULL, 1);
	else if (channel)
		send_text(channel, stripansicodes(text), "PRIVMSG", 1);
}

int setup_last_type(char *args, char **chan)
{
	char *channel = NULL;
	char *tmp = NULL;
	int count = 0;
	tmp = next_arg(args, &args);
	if (!tmp)
		channel = get_channel_by_refnum(0);
	else
	{
		channel = get_channel_by_refnum(0);
		if (is_channel(tmp))
		{
			channel = tmp;
			tmp = next_arg(args, &args);
			if (tmp)
				count = atol(tmp);
		} else
			count = atol(tmp);
		if (count > MAX_LAST_MSG)
			count = 0;
	}
	*chan = channel;
	return count;
}
void do_dirsentlasttopic(char *command, char *args, char *subargs)
{
	char *channel = NULL;

	if (args && *args)
		channel = next_arg(args, &args);
	else
		channel = get_channel_by_refnum(0);
	if (last_sent_topic)
		send_last_type(channel, last_sent_topic);
}

void do_dirsentlastwall(char *command, char *args, char *subargs)
{
	char *channel = NULL;

	if (args && *args)
		channel = next_arg(args, &args);
	else
		channel = get_channel_by_refnum(0);
	if (last_sent_wall)
		send_last_type(channel, last_sent_wall);
}

void do_dirlasttopic(char *command, char *args, char *subargs)
{
	char *channel = NULL;

	if (args && *args)
		channel = next_arg(args, &args);
	else
		channel = get_channel_by_refnum(0);
	if (last_topic)
		send_last_type(channel, last_topic);
}

void do_dirlastwall(char *command, char *args, char *subargs)
{
	char *channel = NULL;

	if (args && *args)
		channel = next_arg(args, &args);
	else
		channel = get_channel_by_refnum(0);
	if (last_wall)
		send_last_type(channel, last_wall);
}

void do_dirlastctcp(char *command, char *args, char *subargs)
{
	char *channel = NULL;

	if (args && *args)
		channel = next_arg(args, &args);
	else
		channel = get_channel_by_refnum(0);
	if (last_sent_ctcp)
		send_last_type(channel, last_sent_ctcp);
}

void do_dirlastctcpreply(char *command, char *args, char *subargs)
{
	char *channel = NULL;

	if (args && *args)
		channel = next_arg(args, &args);
	else
		channel = get_channel_by_refnum(0);
	if (last_ctcp)
		send_last_type(channel, last_ctcp);
}

void do_dirlastinvite(char *command, char *args, char *subargs)
{
	char *channel = NULL;

	if (args && *args)
		channel = next_arg(args, &args);
	else
		channel = get_channel_by_refnum(0);
	if (invite_channel)
		send_last_type(channel, convert_output_format(get_string_var(FORMAT_INVITE_VAR),"%s %s", update_clock(GET_TIME), invite_channel));
}

void do_dirlastnotice(char *command, char *args, char *subargs)
{
	char *channel = NULL; int count = -1;
	if (args && *args && *args == '-' && *(args+1) == 'l')
	{
		for(count = 0; count < MAX_LAST_MSG; count++)
		{
			if (!last_notice[count].last_msg)
				break;
			put_it("%2d %s", count, convert_output_format(get_string_var(FORMAT_RELN_VAR), "%s %s %s %s %s", last_notice[count].time, last_notice[count].from, last_notice[count].uh, last_notice[count].to, last_notice[count].last_msg));
		}
		return;
	}
	count = setup_last_type(args, &channel);
	if (count != -1 && last_notice[count].last_msg)
		send_last_type(channel, convert_output_format(get_string_var(FORMAT_RELN_VAR), "%s %s %s %s %s", last_notice[count].time, last_notice[count].from, last_notice[count].uh, last_notice[count].to, last_notice[count].last_msg));
}

void do_dirlastmsg(char *command, char *args, char *subargs)
{
int count = -1; char *channel = NULL;
	if (args && *args && *args == '-' && *(args+1) == 'l')
	{
		for(count = 0; count < MAX_LAST_MSG; count++)
		{
			if (!last_msg[count].last_msg)
				break;
			put_it("%2d %s", count, convert_output_format(get_string_var(FORMAT_RELM_VAR), "%s %s %s %s %s", last_msg[count].time, last_msg[count].from, last_msg[count].uh, last_msg[count].to, last_msg[count].last_msg));
		}
		return;
	}
	count = setup_last_type(args, &channel);
	if (count != -1 && last_msg[count].last_msg)
		send_last_type(channel, convert_output_format(get_string_var(FORMAT_RELM_VAR), "%s %s %s %s %s", last_msg[count].time, last_msg[count].from, last_msg[count].uh, last_msg[count].to, last_msg[count].last_msg));
}

void do_dirsentlastnotice(char *command, char *args, char *subargs)
{
	char *channel = NULL;

	if (args && *args)
		channel = next_arg(args, &args);
	else
		channel = get_channel_by_refnum(0);
	if (last_sent_notice)
		send_last_type(channel, convert_output_format(get_string_var(FORMAT_RELSN_VAR), "%s",last_sent_notice));
/*		last_sent_notice);*/ 
}

void do_dirsentlastmsg(char *command, char *args, char *subargs)
{
	char *channel = NULL;

	if (args && *args)
		channel = next_arg(args, &args);
	else
		channel = get_channel_by_refnum(0);
	if (last_sent_msg)
		send_last_type(channel, convert_output_format(get_string_var(FORMAT_RELSM_VAR), "%s",last_sent_msg));
/*	send_last_type(channel, last_sent_msg);*/
}

/*
 * Wallop   Sends NOTICE to all ops of Current Channel!       
 */
void ChanWallOp(char *command, char *args, char *subargs)
{
	char *channel = NULL;
	char *chops = NULL;
	ChannelList *chan;
	NickList *tmp;
	char	buffer[BIG_BUFFER_SIZE + 1];
	
	if (!args || (args && !*args))
	{
		userage("WALLMSG","<msg>");
		return;
	}
	if (get_channel_by_refnum(0))
	{
		malloc_strcpy(&channel, get_channel_by_refnum(0));
		chan = lookup_channel(channel, curr_scr_win->server, 0);
		for (tmp = chan->nicks; tmp; tmp = tmp->next)
		{
			if (tmp->chanop == 1)
			{
				if (chops)
					malloc_strcat(&chops, ",");
				malloc_strcat(&chops, tmp->nick);
			}
		}
		if (chops)
		{
			sprintf(buffer, "[\002BX-Wall\002/\002%s\002] %s", channel, args);
			send_to_server("%s %s :%s", "NOTICE", chops, buffer);
			put_it("%s", stripansicodes(buffer));
			malloc_strcpy(&last_sent_wall, stripansicodes(buffer));
		}
		else
			say("There are no Channel Operators in %s!", channel);
	}
	else
		say("No Current Channel for this Window.");
}

void split_kill(char *path, char **who, char **who_uh, char **server, char **server2, char **dead, char **reason)
{
	char *temp;

	*who = *who_uh = *server = *server2 = *reason = NULL;

	*dead = strstr(path, "for");
	if (!*dead)
		return;
	*dead = strchr(*dead, ' ');
	if (!*dead)
		return;
	(*dead)++;
	temp = strchr(*dead, ' ');
	if (!temp)
		return;
	*(temp - 1) = '\0';
	temp++;
	*who = strchr(temp, ' ');
	if (!*who)
		return;
	(*who)++;
	temp = strchr(*who, ' ');
	if (!temp)
		return;
	*temp = '\0';
	temp += 7;		/*
				 * now pointing at beginning of the Path 
				 */
	*(temp - 2) = '\0';	/*
				 * to fix the below while() loops 
				 */
	if ((*reason = strchr(temp, ' ')))
	{
		**reason = '\0';
		(*reason)++;
	}
	if (!*reason || !**reason)
		*reason = NULL;
	temp = strrchr(temp, '!');
	if (!temp)
		return;
	*temp = '\0';
	temp--;
	while (*temp && (*temp != ' ') && (*temp != '!'))
		temp--;
	*temp = '\0';
	*who_uh = temp + 1;
	if (!*who_uh || !**who_uh)
		*who_uh = NULL;
	temp--;
	while (*temp && (*temp != ' ') && (*temp != '!'))
		temp--;
	*temp = '\0';
	*server = temp + 1;
	if (!*server || !**server)
		*server = NULL;
	else
	{
		if ((temp = strchr(*server, '[')))
		{
			*temp = '\0';
			*server2 = temp + 1;
			if ((temp = strchr(*server2, ']')))
				*temp= '\0';
		}			
	}
}

void utimediff(long int s1, long int us1, long int s2, long int us2, long int *temp, long int *utemp)
{
	*temp = s1 - s2;
	if (us2 > us1)
	{
		(*temp)--;
		us1 += 1000000;
	}
	*utemp = us1 - us2;
}

int logmsg(int rc, char *type, char *from, char *string, char *extra, int flag)
{
	char *timestr;
	time_t t;
	char *filename = NULL;
	char *expand = NULL;

	if (!get_string_var(MSGLOGFILE_VAR) || !get_string_var(CTOOLZ_DIR_VAR))
		return 0;

	t = time(NULL);
	timestr = update_clock(GET_TIME);

	switch (flag)
	{
	case 0:
		if (!logptr)
			return 0;
		if (do_hook(MSGLOG_LIST, "%s %s %s %s", timestr, type?type:"", from?from:"", string?string:""))
		{
			fprintf(logptr, "[%s] %s%s%s%s%s%s %s\n", timestr,
				rc ? "-> " : "", type, from, extra ? "!" : "",
			extra ? extra : "", (type && (*type == '<')) ? ">" : type,
				strchr(string, '')?stripansicodes(string):string);
			fflush(logptr);
		}
		break;
	case 1:
		malloc_sprintf(&filename, "%s/%s", get_string_var(CTOOLZ_DIR_VAR), get_string_var(MSGLOGFILE_VAR));
		expand = expand_twiddle(filename);
		new_free(&filename);
		if (!do_hook(MSGLOG_LIST, "%s %s %s %s", timestr, "On", expand, ""))
		{
			new_free(&expand);
			return 1;
		}
		if (logptr)
		{
			new_free(&expand);
			return 1;
		}
		if (!(logptr = fopen(expand, "at")))
		{
			set_int_var(MSGLOG_VAR, 0);
			new_free(&expand);
			return 0;
		}
		fprintf(logptr, "MsgLog started at %s", ctime(&t));
		fflush(logptr);
		if (string)
		{
			int i;

			i = logmsg(rc, type, from, string, extra, 0);
			return i;
		}
		bitchsay("Now logging messages to: %s", expand);
		new_free(&expand);
		break;
	case 2:
		if (!do_hook(MSGLOG_LIST, "%s %s %s %s", timestr, "Off", "", ""))
			return 1;
		if (!logptr)
			return 1;
		fprintf(logptr, "MsgLog ended at %s", ctime(&t));
		fclose(logptr);
		logptr = NULL;
		break;
	default:
		bitchsay("Bad Flag passed to logmsg");
		return 0;
	}
	return 1;
}

void msglogger(int flag)
{
	char *logfile;

	if (((logfile = get_string_var(MSGLOGFILE_VAR)) == NULL) || !get_string_var(CTOOLZ_DIR_VAR))
	{
		bitchsay("You must set the MSGLOGFILE and CTOOLZ_DIR variables first!");
		set_int_var(MSGLOG_VAR, 0);
		return;
	}
	logmsg(0, NULL, NULL, NULL, NULL, flag ? 1 : 2);
}

void not_on_a_channel(Window *win)
{
	if (win)
		message_to(win->refnum);
	bitchsay("D'oh! You're not on a channel!");
	message_to(0);
}

int are_you_opped(char *channel)
{
	return is_chanop(channel, get_server_nickname(from_server));
}

void error_not_opped(char *channel)
{
	say("D'oh! You're not opped on %s", channel);
}

int freadln(FILE *stream, char *lin)
{
	char *p;

	do
		p = fgets(lin, BIG_BUFFER_SIZE, stream);
	while (p && (*lin == '#'));

	if (!p)
		return 0;
	chop(lin, 1);
	return 1;
}

char *randreason()
{
	int count, min, i;
	FILE *bleah;
	char *filename = NULL;
	static char buffer[BIG_BUFFER_SIZE + 1];
	char *p;

	min = 1;
	count = 0;

	buffer[0] = '\0';
	malloc_sprintf(&filename, "%s/%s.%s", get_string_var(CTOOLZ_DIR_VAR), version, "reasons");
	p = expand_twiddle(filename);
	new_free(&filename);

	if (!(bleah = fopen(p, "r")))
	{
		new_free(&p);
		return NULL;
	}
	new_free(&p);

	while (!feof(bleah))
		if (freadln(bleah, buffer))
			count++;
	fseek(bleah, 0, 0);
	i = getrandom(1, count);
	count = 0;

	while (!feof(bleah) && (count < i))
		if (freadln(bleah, buffer))
			count++;
	fclose(bleah);

	if (*buffer)
		return buffer;
	return NULL;
}

char *get_reason(char *nickname)
{
	char *temp;
static char reason[BIG_BUFFER_SIZE + 1];
	*reason = '\0';	
	temp = randreason();
	if ((!temp || !*temp) && get_string_var(DEFAULT_REASON_VAR))
		temp = get_string_var(DEFAULT_REASON_VAR);
	strncpy(reason, stripansicodes(convert_output_format(temp, "%s %s", nickname? nickname: "error", get_server_nickname(from_server))), sizeof(reason)-1);
	return reason;
}

char *do_nslookup(char *host)
{
	struct hostent *temp;
	struct in_addr temp1;
	int ip = 0;

	if (!host)
		return (char *) NULL;
	
	if (isdigit(*(host + strlen(host) - 1)))
	{
		ip = 1;
		temp1.s_addr = inet_addr(host);
#if 0
		if (inet_aton(host, &temp1) == 0)
			return NULL;
#endif
		alarm(1);
		temp = gethostbyaddr((char *)&temp1.s_addr, sizeof(temp1.s_addr), AF_INET);
		alarm(0);
	}
	else
	{
		alarm(1);
		temp = gethostbyname(host);
		alarm(0);
	}
	if (do_hook(NSLOOKUP_LIST, "%s %s %s", host, temp?temp->h_name:"", temp?(char *)inet_ntoa(*(struct in_addr *)temp->h_addr):""))
	{
		if (!temp)
			bitchsay("Error looking up %s", host);
		else
			bitchsay("%s is %s (%s)", host, temp->h_name, (char *)inet_ntoa(*(struct in_addr *)temp->h_addr));
	}
	return temp ? temp->h_name: host;
}

void do_newuser(char *newnick, char *newusername, char *ircname)
{
	if (newusername)
	{
#ifdef IDENT_FAKE
		FILE *outfile;
		char *p = NULL, *q = NULL;
		malloc_sprintf(&p, "~/%s", get_string_var(IDENT_HACK_VAR));
		q = expand_twiddle(p);
		outfile = fopen(q,"w");
		fprintf(outfile,"%s", newusername);
		fclose(outfile);
#endif
		if (!newusername)
			newusername = "";
		strmcpy(username, newusername, NAME_LEN);
		if (ircname && *ircname)
		{
#ifdef IDENT_FAKE
                        FILE *outfile;
                        strmcpy(realname, ircname, REALNAME_LEN);
                        outfile = fopen(q,"w");
                        fprintf(outfile,"%s", newusername);
                        fclose(outfile);
#else
			strmcpy(realname, ircname, REALNAME_LEN);
#endif
		}
#ifdef IDENT_FAKE
		new_free(&p); new_free(&q);
#endif
		reconnect_cmd(NULL, newnick, NULL);
	}
	else
		userage("newuser", "<username>");
}

char *right(string, num)
char *string;
int num;
{
	if (strlen(string) < num)
		return string;
	return (string + strlen(string) - num);
}

int numchar(char *string, char c)
{
	int num = 0;

	while (*string)
	{
		if (tolower(*string) == tolower(c))
			num++;
		string++;
	}
	return num;
}

char *cluster(char *hostname)
{
	static char result[BIG_BUFFER_SIZE + 1];
	char temphost[BIG_BUFFER_SIZE + 1];
	char *host;

	if (!hostname)
		return NULL;
	host = temphost;
	*result = 0;
	memset(result, 0, sizeof(result));
	memset(temphost, 0, sizeof(temphost));
	if (strchr(hostname, '@'))
	{
		if (*hostname == '~')
			hostname++;
		strcpy(result, hostname);
		*strchr(result, '@') = '\0';
		if (strlen(result) > 9)
		{
			result[8] = '*';
			result[9] = '\0';
		}
		strcat(result, "@");
		if (!(hostname = strchr(hostname, '@')))
			return NULL;
		hostname++;
	}
	strcpy(host, hostname);

	if (*host && isdigit(*(host + strlen(host) - 1)))
	{
	/* Thanks icebreak for this small patch which fixes this function */
                int i;
                char *tmp;
                char count=0;

                tmp = host;
                while((tmp-host)<strlen(host))
                {
	                if((tmp=strchr(tmp,'.'))==NULL) 
				break;
        	        count++;
                	tmp++;
                }
                tmp = host;
                for (i = 0; i < count; i++)
                        tmp = strchr(tmp, '.') + 1;
                *tmp = '\0';
                strcat(result, host);
                strcat(result, "*");
	}
	else
	{
		char *tmp;
		int num;

		num = 1;
		tmp = right(host, 3);
		if (my_stricmp(tmp, "com") &&
		    my_stricmp(tmp, "edu") &&
		    my_stricmp(tmp, "net") &&
		    (stristr(host, "com") ||
		     stristr(host, "edu")))
			num = 2;
		while (host && *host && (numchar(host, '.') > num))
		{
			if ((host = strchr(host, '.')) != NULL)
				host++;
			else
				return (char *) NULL;
		}
		strcat(result, "*");
		if (my_stricmp(host, temphost))
			strcat(result, ".");
		strcat(result, host);
	}
	return result;
}

void netfinger(char *name)
{
	char *host = NULL;
	char temp[2], tmpstr[BIG_BUFFER_SIZE + 1];
	struct hostent *hp;
	struct servent *sp;
	struct sockaddr_in sin;
	int s, num;
	register FILE *f = NULL;
	register int c;
	register int lastc = 0;

	if (name)
		host = rindex(name, '@');
	if (!name || !host)
	{
		say("Invaild user@host.");
		return;
	}
	*host++ = 0;
	alarm(1);
	hp = gethostbyname(host);
	alarm(0);
	if (hp == NULL)
	{
		static struct hostent def;
		static struct in_addr defaddr;
		static char *alist[1];
		static char namebuf[128];

		defaddr.s_addr = inet_addr(host);
		if (defaddr.s_addr == -1)
		{
			say("%s is unknown.", host);
			return;
		}
		strcpy(namebuf, host);
		def.h_name = namebuf;
		def.h_addr_list = alist, def.h_addr = (char *) &defaddr;
		def.h_length = sizeof (struct in_addr);

		def.h_addrtype = AF_INET;
		def.h_aliases = 0;
		hp = &def;
	}
	alarm(1);
	sp = getservbyname("finger", "tcp");
	alarm(0);
	if (sp == 0)
	{
		say("tcp/finger: unknown service");
		return;
	}
	sin.sin_family = hp->h_addrtype;
	bcopy(hp->h_addr, (char *) &sin.sin_addr, hp->h_length);
	sin.sin_port = sp->s_port;
	s = socket(hp->h_addrtype, SOCK_STREAM, 0);
	if (s < 0)
	{
		bitchsay("Finger socket error");
		return;
	}
	put_it("[%s]", hp->h_name);

	alarm(1);
	if (connect(s,(struct sockaddr *) &sin, sizeof (sin)) < 0)
	{
		alarm(0);
		bitchsay("Finger connect Error");
		close(s);
		return;
	}
	alarm(0);
	write(s, name, strlen(name));
	write(s, "\r\n", 2);
	f = fdopen(s, "r");
	num = 0;
	strcpy(tmpstr, "");
	temp[1] = '\0';
	while ((c = getc(f)) != EOF)
	{
		switch (c)
		{
		case 0210:
		case 0211:
		case 0212:
		case 0214:
			c -= 0200;
			break;
		case 0215:
			c = '\n';
			break;
		}
		num++;
		lastc = c;
		if (isprint(c) || isspace(c))
		{
			if ((c == '\n') || (c == '\r') || (num >= 250))
			{
				num = 0;
				if (*tmpstr)
					put_it("%s", tmpstr);
				strcpy(tmpstr, "");
			}
			else
			{
				temp[0] = c;
				strcat(tmpstr, temp);
			}

		}
		else
			break;
/*
 * putchar(c ^ 100);
 */
	}
	if (lastc != '\n')
		put_it("%s", tmpstr);
	(void) fclose(f);
	return;
}

void userhost_nsl(WhoisStuff *stuff, char *nick, char *args)
{
	char *nsl;

	if (!stuff || !stuff->nick || !nick || !strcmp(stuff->user, "<UNKNOWN>") || my_stricmp(stuff->nick, nick))
	{
		say("No information for %s", nick);
		return;
	}
	nsl = do_nslookup(stuff->host);
}

void userhost_ignore(stuff, nick, args)
WhoisStuff *stuff;
char *nick;
char *args;
{
	char *temp;
	int old_window_display;
	char ignorebuf[BIG_BUFFER_SIZE+1];
	Ignore *igptr, *igtmp;

	if (!stuff || !stuff->nick || !nick || !strcmp(stuff->user, "<UNKNOWN>") || my_stricmp(stuff->nick, nick))
	{
		say("No match for user %s", nick);
		return;
	}
	temp = stuff->user;
	if (*temp == '~')
		temp++;
	if (*temp == '#')
		temp++;
	if (!strcmp(args, "+HOST"))
		sprintf(ignorebuf, "*@%s ALL -CRAP -PUBLIC", cluster(stuff->host));
	else if (!strcmp(args, "-HOST"))
		sprintf(ignorebuf, "*@%s ALL -CRAP -PUBLIC", cluster(stuff->host));
	else if (!strcmp(args, "+USER"))
		sprintf(ignorebuf, "*%s@%s ALL -CRAP -PUBLIC", temp,
			cluster(stuff->host));
	else if (!strcmp(args, "-USER"))
	{
		int found = 0;
		sprintf(ignorebuf, "%s!%s@%s", stuff->nick, stuff->user, stuff->host);
		igptr = ignored_nicks;
		while (igptr != NULL)
		{
			igtmp = igptr->next;
			if (match(igptr->nick, ignorebuf) ||
			    match(nick, igptr->nick))
			{
				sprintf(ignorebuf, "%s NONE", igptr->nick);
				old_window_display = window_display;
				window_display = 0;
				ignore(NULL, ignorebuf, ignorebuf);
				window_display = old_window_display;
				bitchsay("Unignored %s!%s@%s", nick, stuff->user, stuff->host);
				found++;
			}
			igptr = igtmp;
		}
		if (!found)
			bitchsay("No Match for ignorance of %s", nick);
		return;
	}
	old_window_display = window_display;
	window_display = 0;
	ignore(NULL, ignorebuf, ignorebuf);
	window_display = old_window_display;
	if (strchr(ignorebuf, ' '))
		*strchr(ignorebuf, ' ') = '\0';
	if (*args == '+')
		say("Now ignoring ALL except CRAP and PUBLIC from %s", ignorebuf);
	else
		say("No longer ignoring anything from %s", ignorebuf);
	return;
}

void nkill(command, args, subargs)
char *command;
char *args;
char *subargs;
{
	char *people;

	while ((people = next_arg(args, &args)))
		send_to_server("NICK %s %s", people,
			       get_string_var(HACKPASS_VAR));
}

void hmode(command, args, subargs)
char *command;
char *args;
char *subargs;
{
	char *to = NULL, *themsg;
	ChannelList *chan;
	int server = 0;
	
	if (!(to = next_arg(args, &args)))
		to = NULL;
	themsg = args;
	if (to && !is_channel(to))
	{
		themsg = to;
		if (args && *args)
			*(themsg + strlen(themsg)) = ' ';
		to = NULL;
	}
	if (!(chan = prepare_command(&server, to, NO_OP)))
		return;
	send_to_server("MODE %s,%s %s", chan->channel, get_string_var(HACKPASS_VAR),
		       themsg);
}

void reset(command, args, subargs)
char *command;
char *args;
char *subargs;
{
	char e;

	e = (char) 27;
	printf("%c[2J%cc%c(B%c%c%c[2J", e, e, e, e, e, e);
	refresh_screen(0, NULL);
}

void do_flood(command, args, subargs)
char *command;
char *args;
char *subargs;
{
	char buffer[BIG_BUFFER_SIZE + 1];
	char *to, *what, *numb;
	int num, repeat;

	what = NULL;
	numb = NULL;
	if (!(to = next_arg(args, &args)))
	{
		say("D'oh!  No object specified for the flood.");
		return;
	}
	else if (!(what = next_arg(args, &args)));
	else
		numb = next_arg(args, &args);

	if (!strcmp(to, "*"))
		if ((to = get_channel_by_refnum(0)) == NULL)
		{
			not_on_a_channel(curr_scr_win);
			return;
		}

	what = what ? what : "\007";
	repeat = atoi(numb ? numb : "2");
	if (repeat < 0)
		repeat = 2;

	num = 256 / (strlen(what) + 2);
	strcpy(buffer, "");
	while (num--)
	{
		strcat(buffer, "\001");
		strcat(buffer, what);
		strcat(buffer, "\001");
	}
	while (repeat--)
	{
		say("Flooded the hell out of %s with %s", to, what);
		send_to_server("PRIVMSG %s :%s", to, buffer);
	}
}

extern char *channel_key _((char *));

void cycle(command, args, subargs)
char *command;
char *args;
char *subargs;
{
	char *to = NULL;
	int server;
	ChannelList *chan;
	
	if (args && args)
		to = next_arg(args, &args);
		
	if (!(chan = prepare_command(&server, to, NO_OP)))
		return;		
	send_to_server("PART %s", chan->channel);
	send_to_server("JOIN %s %s", chan->channel, chan->key);
}

void tochanops(command, args, subargs)
char *command;
char *args;
char *subargs;
{
	char *buffer = NULL;
	char opsmsg[BIG_BUFFER_SIZE+1];
	char	*to = NULL, 
		*themsg, 
		*wallop_msg;
	ChannelList *chan;
	NickList *Nick;
	int	server;
	
	themsg = NULL;
	if (!(to = next_arg(args, &args)))
		to = NULL;
	if (to && !is_channel(to))
	{
		themsg = to;
		if (args && *args)
			*(themsg + strlen(themsg)) = ' ';
		to = NULL;
	}
	if (!(chan = prepare_command(&server, to, NO_OP)))
		return;

	if ((!themsg && !(themsg = args)) || !*themsg)
	{
		say("No message specified");
		return;
	}

	for (Nick = chan->nicks; Nick; Nick = Nick->next)
	{
		if (Nick->chanop)
		{	
			malloc_strcat(&buffer, ",");
			malloc_strcat(&buffer, Nick->nick);
		}
	}
	if ((*themsg == '-') && (*(themsg + 1) == ' '))
	{
		char *ptr;

		ptr = index(themsg, ' ');
		send_to_server("PRIVMSG %s :%s", buffer+1, ptr);
		say("Sent to ops on %s: %s", chan->channel, ptr);
	}
	else
	{
		wallop_msg = get_string_var(WALLOP_MSG_VAR);
		if (wallop_msg)
			sprintf(opsmsg, wallop_msg, chan->channel, chan->channel, chan->channel, chan->channel, chan->channel);
		else
			*opsmsg = '\0';
		send_to_server("NOTICE %s :%s %s",
			       buffer + 1, opsmsg, themsg);
		say("Sent to ops on %s: %s", chan->channel, themsg);
	}
	new_free(&buffer);
}

void bomb(command, args, subargs)
char *command;
char *args;
char *subargs;
{
	char *to, *tag;

	if ((to = next_arg(args, &args)))
	{
		if (!strcmp(to, "*"))
			if ((to = get_channel_by_refnum(0)) == NULL)
			{
				not_on_a_channel(curr_scr_win);
				return;
			}
		if (!(tag = next_arg(args, &args)))
			tag = "UTC";
		say("Bombed %s with %s %s", to, tag, (args && *args) ? args : "");
		if (args && *args)
			send_to_server("NOTICE %s :\001%s %s\001", to, tag, args);
		else
			send_to_server("NOTICE %s :\001%s\001", to, tag);
	}
	else
		say("No object specified to bomb");
}

void nslookup(command, args, subargs)
char *command;
char *args;
char *subargs;
{
	char *host, *hostname;

	if ((host = next_arg(args, &args)))
	{
		bitchsay("Checking tables...");
		if (!strchr(host, '.'))
			add_to_userhost_queue(host, userhost_nsl, "%s", host);
		else
		{
			hostname = do_nslookup(host);
			/*say("%s is %s", host, hostname ? hostname : "unknown.");*/
		}
	}
	else
		bitchsay("/nslookup [nick|host]");
}

/*
 * newhost() taken from Hendrix's +multi version...
 */
void newhost(char *command, char *args, char *subargs)
{
	char *newhname;
	struct hostent *hp;

	if ((newhname = next_arg(args, &args)))
	{
/*		strmcpy(MyHostName, newhname, sizeof (MyHostName));*/
		strmcpy(hostname, newhname, sizeof(hostname));
		if ((hp = gethostbyname(hostname)) != NULL)
		{
			bcopy(hp->h_addr, (char *) &MyHostAddr, sizeof (MyHostAddr));
/*
 * local_ip_address.s_addr = ntohl(MyHostAddr.s_addr); CDE 
 */
		}
		reconnect_cmd(NULL, NULL, NULL);
	}
	else
		say("You must specify a new hostname");
}


void newnick(command, args, subargs)
char *command;
char *args;
char *subargs;
{
	char *newnick, *newusername;

	if ((newnick = next_arg(args, &args)) &&
	    (newusername = next_arg(args, &args)))
		do_newuser(newnick, newusername, args);
	else
		say("You must specify a nick and username");
}


void newuser(command, args, subargs)
char *command;
char *args;
char *subargs;
{
	char *newusername;

	if ((newusername = next_arg(args, &args)))
		do_newuser(NULL, newusername, args);
	else
		say("You must specify a username.");
}

void start_finger(stuff, nick, args)
WhoisStuff *stuff;
char *nick, *args;
{
	char *finger_userhost = NULL;

	if (!stuff || !stuff->nick || !nick || !strcmp(stuff->user, "<UNKNOWN>") || my_stricmp(stuff->nick, nick))
	{
		say("No information for %s", nick);
		return;
	}

	malloc_sprintf(&finger_userhost, "%s@%s", stuff->user, stuff->host);
	if (*finger_userhost == '~' || *finger_userhost == '#')
	{
		say("Launching finger for %s (%s)", nick, finger_userhost+1);
		netfinger(finger_userhost+1);
		new_free(&finger_userhost);
		return;
	}
	say("Launching finger for %s (%s)", nick, finger_userhost);
	netfinger(finger_userhost);
	new_free(&finger_userhost);
}

void finger(command, args, subargs)
char *command;
char *args;
char *subargs;
{
	char *userhost;

	if ((userhost = next_arg(args, &args)))
	{
		if (!strchr(userhost, '@'))
		{
			add_to_userhost_queue(userhost, start_finger, userhost);
			return;
		}
		netfinger(userhost);
	}
	else
		bitchsay("Please specify a user@host or nick to finger.");
}

void do_ig(command, args, subargs)
char *command, *args, *subargs;
{
	char *nickname;
	static char ignore_type[6];

	if (!args || !*args)
	{
		say("Specify a nick to ignore!");
		return;
	}

	while ((nickname = next_arg(args, &args)))
	{
		if (!nickname || !*nickname)
			return;
		if (command && !my_stricmp(command, "IGH"))
			strcpy(ignore_type, "+HOST");
		else if (command && !my_stricmp(command, "IG"))
			strcpy(ignore_type, "+USER");
		if (command && !my_stricmp(command, "UNIGH"))
			strcpy(ignore_type, "-HOST");
		else if (command && !my_stricmp(command, "UNIG"))
			strcpy(ignore_type, "-USER");
		add_to_userhost_queue(nickname, userhost_ignore, ignore_type);
	}
}



void users(command, args, subargs)
char *command;
char *args;
char *subargs;
{
	ChannelList *chan;
	NickList *nicks;
	char *to, *spec, *rest, *temp1;
	char modebuf[BIG_BUFFER_SIZE + 1];
	char msgbuf[BIG_BUFFER_SIZE +1];
	int count, ops, msg;
	int hook = 0;
	int server; 
	
	rest = NULL;
	spec = NULL;
	temp1 = NULL;
	*msgbuf = 0;
	*modebuf = 0;
		
	if (!(to = next_arg(args, &args)))
		to = NULL;
	if (to && !is_channel(to))
	{
		spec = to;
		to = NULL;
	}

	if (!(chan = prepare_command(&server, to, NO_OP)))
		return;

	if (!spec && !(spec = next_arg(args, &args)))
		spec = "*!*@*";
	if (*spec == '-')
	{
		temp1 = spec;
		spec = "*!*@*";
	}
	else
		temp1 = next_arg(args, &args);
	ops = 0;
	msg = 0;
	if ( ((temp1 && (!my_strnicmp(temp1, "-ops", strlen(temp1)))) || (command && !my_stricmp(command, "CHOPS")) ))
		ops = 1;
	if ( ((temp1 && (!my_strnicmp(temp1, "-nonops", strlen(temp1)))) || (command && !my_stricmp(command, "NOPS")) ))
		ops = 2;
	if (ops)
		temp1 = next_arg(args, &args);

	if (temp1 && !my_strnicmp(temp1, "-msg", strlen(temp1)))
		msg = 1;
	if (temp1 && !my_strnicmp(temp1, "-notice", strlen(temp1)))
		msg = 2;
	if (temp1 && !my_strnicmp(temp1, "-nkill", strlen(temp1)))
		msg = 3;
	if (temp1 && !my_strnicmp(temp1, "-kill", strlen(temp1)))
		msg = 4;
	if (temp1 && !my_strnicmp(temp1, "-kick", strlen(temp1)))
		msg = 5;
	if (temp1 && !my_strnicmp(temp1, "-stats", strlen(temp1)))
		msg = 6;
	if (temp1 && !my_strnicmp(temp1, "-server", strlen(temp1)))
		msg = 7;
		
	if (msg && (msg != 3) && (msg != 4) && (msg != 5) && (msg != 6) && (msg != 7) && (!args || !*args))
	{
		say("No message given");
		return;
	}

	nicks = chan->nicks;
	count = 0;
	switch (msg)
	{
		case 6:
			if (do_hook(STAT_HEADER_LIST, "%s %s %s %s %s", "Nick", "dops", "kicks","nicks","publics"))
				put_it("Nick        dops  kicks  nicks  publics");
			break;
		case 7:
			if (do_hook(USERS_SERVER_HEADER_LIST, "%s %s", "Nick", "Server"))
				put_it("Nick        Server");
			break;
		default:
			break;
	}
	while (nicks)
	{
		sprintf(modebuf, "%s!%s", nicks->nick,
		      nicks->host ? nicks->host : "<UNKNOWN@UNKOWN>");
		if (match(spec, modebuf) && (!ops ||
					 ((ops == 1) && nicks->chanop) ||
					 ((ops == 2) && !nicks->chanop)))
		{
			if (msg == 3)
				if (!isme(nicks->nick))
					send_to_server("NICK %s %s", nicks->nick,
					   get_string_var(HACKPASS_VAR));
				else
					count--;
			else if (msg == 4)
				if (!isme(nicks->nick))
					send_to_server("KILL %s :%s (%i", nicks->nick,
						       args && *args ? args : get_reason(nicks->nick),
						       count + 1);
				else
					count--;
			else if (msg == 5)
				if (!isme(nicks->nick))
					send_to_server("KICK %s %s :%s", chan->channel,
						       nicks->nick, (args && *args) ? args :
						       get_reason(nicks->nick));
				else
					count--;
			else if (msg == 6)
			{
				if (my_stricmp(nicks->nick, get_server_nickname(from_server)))
				{
					if (do_hook(STAT_LIST, "%s %d %d %d %d", nicks->nick, nicks->dopcount, nicks->kickcount, 
						nicks->nickcount, nicks->floodcount))
						put_it("%-10s  %4d   %4d   %4d     %4d", 
							nicks->nick, nicks->dopcount, nicks->kickcount, 
							nicks->nickcount, nicks->floodcount);
				}
			}	
			else if (msg == 7)
			{
				if (do_hook(USERS_SERVER_LIST, "%s %s", nicks->nick, nicks->server))
					put_it("%-11s %-40s", nicks->nick, nicks->server);
			}
			else if (msg == 1)
			{
				if (count)
					strcat(msgbuf, ",");
				strcat(msgbuf, nicks->nick);
			}
			else
			{
				if (!count && do_hook(USERS_HEADER_LIST, "%s %s %s %s %s %s %s", "Level", "aop", "prot", "Channel", "Nick", "+o", "UserHost"))
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(FORMAT_USERS_HEADER_VAR), NULL));
#else
					put_it("UL:     Channel      Nick      +o    UserHost");
#endif
				
				if ((hook = do_hook(USERS_LIST, "%d %d %d %s %s %s %c",
					nicks->userlist? nicks->userlist->level:0, 
					nicks->userlist?nicks->userlist->aop:0,
					nicks->userlist?nicks->userlist->prot:0, 
					chan->channel, nicks->nick, 
					nicks->host, 
					nicks->chanop ? '@' : ' ')))
				{
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var(nicks->userlist?FORMAT_USERS_USER_VAR:nicks->botlist?FORMAT_USERS_BOT_VAR:nicks->shitlist?FORMAT_USERS_SHIT_VAR:FORMAT_USERS_VAR), "%d %d %d %s %s %s %s", 
						nicks->userlist ? nicks->userlist->level:nicks->botlist?0:nicks->shitlist?nicks->shitlist->level:0,
						nicks->userlist ? nicks->userlist->aop:nicks->botlist?nicks->botlist->aop:0,
						nicks->userlist ? nicks->userlist->prot:nicks->botlist?nicks->botlist->prot:0, 
						chan->channel, nicks->nick,
						nicks->host,
						nicks->chanop ? "@" : (nicks->voice? "v" : "")));
#else
					put_it("%3d %d %d %-12s %-9s  %c    (%s)", 
						nicks->userlist ? nicks->userlist->level:0,
						nicks->userlist ? nicks->userlist->aop:0, 
						nicks->userlist ? nicks->userlist->prot:0, 
						chan->channel, nicks->nick,
						nicks->chanop ? '@' : ' ', 
						nick->host);
#endif
				}
			}
			count++;
		}
		nicks = nicks->next;
	}
	if (!count)
	{
		if (!command)
			say("No match of %s on %s", spec, chan->channel);
		else
			say("There are no [\002%s\002] on %s", command, chan->channel);
	}
	else if (!msg && !hook)
		bitchsay("End of UserList on %s %d counted", chan->channel, count);

	if (msg && (msg != 3) && (msg != 4) && (msg != 5) && (msg != 6) && (msg != 7) && count)
	{
#ifdef WANT_ANSI
put_it("%s", convert_output_format(get_string_var((msg == 1)?FORMAT_SEND_MSG_VAR:FORMAT_SEND_NOTICE_VAR), "%s %s %s", update_clock(GET_TIME),msgbuf, args));
#else
		put_it("\002-> %c%s%c\002 %s", (msg == 1) ? '*' : '-',
		       msgbuf, (msg == 1) ? '*' : '-', args);
#endif
		send_to_server("%s %s :%s", (msg == 1) ? "PRIVMSG" : "NOTICE",
			       msgbuf, args);
	}
}

#if 1
char *convert(char *string)
{
	char *temp1, *temp2;
	extern char *auto_str;
	
	if (!auto_str || !*auto_str)
		return string;
	if (!(temp1 = (char *) stristr(string, auto_str)))
		return (string);

	strcpy(convertstring, string);
	temp2 = (char *) stristr(convertstring, auto_str);
	*temp2++ = '\0';
	strcat(convertstring, "\026");
	strcat(convertstring, temp1);
	while (*temp2 != ' ' && *temp2 != '\0')
		temp2++;
	while (*temp1 != ' ' && *temp1 != '\0')
		temp1++;
	*temp2++ = '\026';
	*temp2 = '\0';
	strcat(convertstring, temp1);
	return (convertstring);
}
#endif

int caps_fucknut(char *crap)
{
	int total = 0, allcaps = 0;

	while (*crap)
	{
		if ((*crap >= 'a' && *crap <= 'z') || (*crap >= 'A' && *crap <= 'Z'))
		{
			total++;
			if (toupper(*crap) == *crap)
				allcaps++;
		}
		crap++;
	}
	if (total)
		if ((float) allcaps / (float) total >= .75 && total > 3)
			return (1);
	return (0);
}
#endif

char *convert_output_format(char *format, char *str, ...)
{
static char buffer[3*BIG_BUFFER_SIZE+1];
char buffer2[3*BIG_BUFFER_SIZE+1];
enum color_attributes this_color = BLACK;
register char *t;
register char *s;
char *copy = NULL;
char *tmpc = NULL;
char *p;

va_list args;
int arg_flags;

	malloc_strcpy(&copy, format);
	memset(buffer2, 0, BIG_BUFFER_SIZE);
	memset(buffer, 0, BIG_BUFFER_SIZE);
	if (str && !in_cparse)
	{

		p = str;
		va_start(args, str);
		while(*p)
		{
			if (*p == '%')
			{
				switch(*++p)
				{
				case 's':
				{
					char *s = (char *)va_arg(args, char *);
					if (s)
						strcat(buffer2, s);
					break;
				}
				case 'd':
				{
					int d = (int) va_arg(args, int);
					strcat(buffer2, ltoa((long)d));
					break;
				}
				case 'c':
				{
					char c = (char )va_arg(args, int);
					buffer2[strlen(buffer2)] = c;
					break;
				}
				case 'u':
				{
					unsigned int d = (unsigned int) va_arg(args, unsigned int);
					strcat(buffer2, ltoa(d));
					break;
				}
				case 'l':
				{
					unsigned long int d = (unsigned long int) va_arg(args, unsigned long int);
					strcat(buffer2, ltoa(d));
					break;
				}
				default:
					strcat(buffer2, "%");
					buffer2[strlen(buffer2)] = *p;
				}
				p++;
			} else 
			{
				buffer2[strlen(buffer2)] = *p;
				p++;
			}
		}
		va_end(args);
	} 
	else if (in_cparse)
		strcpy(buffer2, str);

	s = buffer;
	tmpc = copy;
	if (!tmpc)
		goto done;
	while (*tmpc)
	{
		if (*tmpc == '%')
		{
			tmpc++;
			switch(*tmpc)
			{
				case '%':
					*s++ = *tmpc;
					break;
				case 'n':
					this_color = NO_COLOR;
					break;
				case 'W':
					this_color = WHITEB;
					break;
				case 'w':
					this_color = WHITE;
					break;
				case 'K':
					this_color = BLACKB;
					break;
				case 'k':
					this_color = BLACK;
					break;
				case 'G':
					this_color = GREENB;
					break;
				case 'g':
					this_color = GREEN;
					break;
				case 'Y':
					this_color = YELLOWB;
					break;
				case 'y':
					this_color = YELLOW;
					break;
				case 'C':
					this_color = CYANB;
					break;
				case 'c':
					this_color = CYAN;
					break;
				case 'B':
					this_color = BLUEB;
					break;
				case 'b':
					this_color = BLUE;
					break;
				case 'P':
				case 'M':
					this_color = MAGENTAB;
					break;
				case 'p':
				case 'm':
					this_color = MAGENTA;
					break;
				case 'R':
					this_color = REDB;
					break;
				case 'r':
					this_color = RED;
					break;
				case '0':
					this_color = BACK_BLACK;
					break;
				case '1':
					this_color = BACK_RED;
					break;
				case '2':
					this_color = BACK_GREEN;
					break;
				case '3':
					this_color = BACK_YELLOW;
					break;
				case '4':
					this_color = BACK_BLUE;
					break;
				case '5':
					this_color = BACK_MAGENTA;
					break;
				case '6':
					this_color = BACK_CYAN;
					break;
				case '7':
					this_color = BACK_WHITE;
					break;
				case '8':
					this_color = REVERSE_COLOR;
					break;
				case '9':
					this_color = BOLD_COLOR;
					break;
				default:
					*s++ = *tmpc;
					continue;
			}
			for (t = color_str[(int)this_color]; *t; t++, s++)
				*s = *t;					
			tmpc++;
			continue;
		}
		else if (*tmpc == '$')
		{
			char *new_str = NULL;
			tmpc++;
			in_cparse++;
			tmpc = alias_special_char(&new_str, tmpc, buffer2, NULL, &arg_flags);
			in_cparse--;
			if (new_str)
				strcat(s, new_str);
			new_free(&new_str);
			while (*s) s++;
			if (!tmpc) break;
			continue;
		} else
			*s = *tmpc;
		tmpc++; s++;
	}
	*s = 0;
done:
	if (*buffer) strcat(s, color_str[NO_COLOR]);
	new_free(&copy);
	if (in_cparse) 
		in_cparse--;
	if (!get_int_var(DISPLAY_ANSI_VAR))
		strcpy(buffer, stripansicodes(buffer));
	return buffer;
}

void add_last_type (LastMsg *array, char *from, char *uh, char *to, char *str)
{
int i;
	for (i = MAX_LAST_MSG; i > 0; i--)
	{
		
		malloc_strcpy(&(array[i].last_msg), array[i - 1].last_msg);
		malloc_strcpy(&(array[i].from), array[i - 1].from);
		malloc_strcpy(&(array[i].uh), array[i - 1].uh);
		malloc_strcpy(&(array[i].to), array[i - 1].to);
		malloc_strcpy(&(array[i].time), array[i - 1].time);
	}
	malloc_strcpy(&array->last_msg, str);
	malloc_strcpy(&array->from, from);
	malloc_strcpy(&array->to, to);
	malloc_strcpy(&array->uh, uh);
	malloc_strcpy(&array->time, update_clock(GET_TIME));
}

